// packages/node/src/index.ts
import { HTMLRewriter } from "@miniflare/html-rewriter";

// packages/shared/utils.ts
import { join } from "node:path";
var GOOGLE_FONTS_URL = "fonts.gstatic.com";
var buildCsp = (policy, handlers) => {
  const { inlineScriptHandler, inlineStyleHandler, scriptHandler, styleHandler } = handlers;
  addToPolicy(policy, "script-src", scriptHandler.hashValues);
  addToPolicy(policy, "script-src", inlineScriptHandler.hashValues);
  addToPolicy(policy, "style-src", inlineStyleHandler.hashValues);
  if (scriptHandler.hashValues || inlineScriptHandler.hashValues) {
    addToPolicy(policy, "script-src", "'strict-dynamic'");
  }
  addToPolicy(policy, "style-src", styleHandler.urls);
  if (inlineScriptHandler.hashValues.length) {
    addToPolicy(policy, "style-src", "'self'");
  }
  if (styleHandler.urls.includes("fonts.googleapis.com")) {
    addToPolicy(policy, "font-src", GOOGLE_FONTS_URL);
  }
  return Object.entries(policy).map(([name, value]) => `${name} ${value.join(" ")}`).join("; ");
};
var addToPolicy = (policy, directive, hashValues) => {
  const hashParts = hashValues.trim().split(/\s+/).filter((s) => s.length);
  if (hashParts.length) {
    if (!policy[directive]) {
      policy[directive] = [];
    }
    for (const hashPart of hashParts) {
      if (!policy[directive].includes(hashPart)) {
        policy[directive].push(hashPart);
      }
    }
  }
};
var resolvePath = (filepath, { base, outDir, root }) => {
  return join(root, outDir, filepath.replace(base, ""));
};

// packages/shared/BaseHandler.ts
class BaseHandler {
  algorithm;
  config;
  FileType;
  hasher;
  hashes = [];
  constructor(algorithm, config, HasherContructor, FileType) {
    this.algorithm = algorithm;
    this.hasher = new HasherContructor(algorithm);
    this.config = config;
    this.FileType = FileType;
  }
  get hashValues() {
    return this.hashes.map((h) => `'${h}'`).join(" ");
  }
  calculateHash = (contentsToHash) => {
    this.hasher.update(contentsToHash);
    const hashed = this.hasher.digest();
    const hash = `${this.algorithm}-${hashed}`;
    this.saveHash(hash);
    return hash;
  };
  getFileContents = async (assetPath) => {
    if (assetPath.startsWith("https://")) {
      const response = await fetch(assetPath);
      return await response.text();
    }
    const filepath = resolvePath(assetPath, this.config);
    const file = new this.FileType(filepath);
    const fileContents = await file.read();
    if (!fileContents) {
      return "";
    }
    return fileContents;
  };
  saveHash = (hash) => {
    if (!this.hashes.includes(hash)) {
      this.hashes.push(hash);
    }
  };
}

// packages/shared/InlineScriptHandler.ts
class InlineScriptHandler extends BaseHandler {
  textContent = "";
  async element(element) {
    if (element.hasAttribute("src")) {
      return;
    }
    this.textContent = "";
  }
  text(chunk) {
    this.textContent += chunk.text;
    if (chunk.lastInTextNode) {
      this.calculateHash(this.textContent);
    }
  }
}

// packages/shared/InlineStyleHandler.ts
class InlineStyleHandler extends BaseHandler {
  textContent = "";
  async element() {
    this.textContent = "";
  }
  text(chunk) {
    this.textContent += chunk.text;
    if (chunk.lastInTextNode) {
      this.calculateHash(this.textContent);
    }
  }
}

// packages/shared/ScriptHandler.ts
var srcAttribute = "src";
var crossOriginAttribute = "crossorigin";
var integrityAttribute = "integrity";

class ScriptHandler extends BaseHandler {
  async element(element) {
    const filePath = element.getAttribute(srcAttribute);
    if (!filePath) {
      return;
    }
    const existingHash = element.getAttribute(integrityAttribute);
    if (existingHash) {
      this.saveHash(existingHash);
      return;
    }
    const contentsToHash = await this.getFileContents(filePath);
    if (!contentsToHash) {
      return;
    }
    const hash = this.calculateHash(contentsToHash);
    element.setAttribute("integrity", hash);
    if (filePath.startsWith("https://") && !element.hasAttribute(crossOriginAttribute)) {
      element.setAttribute(crossOriginAttribute, "anonymous");
    }
  }
}

// packages/shared/StyleHandler.ts
var hrefAttribute = "href";
var crossOriginAttribute2 = "crossorigin";
var integrityAttribute2 = "integrity";
var relAttribute = "rel";

class StyleHandler extends BaseHandler {
  urlCache = [];
  async element(element) {
    const rel = element.getAttribute(relAttribute);
    if (!rel || rel !== "stylesheet") {
      return;
    }
    const existingHash = element.getAttribute(integrityAttribute2);
    if (existingHash) {
      this.saveHash(existingHash);
      return;
    }
    const fileUrl = element.getAttribute(hrefAttribute);
    if (!fileUrl) {
      return;
    }
    if (fileUrl.endsWith(".css")) {
      const contentsToHash = await this.getFileContents(fileUrl);
      if (!contentsToHash) {
        return;
      }
      const hash = this.calculateHash(contentsToHash);
      element.setAttribute("integrity", hash);
    }
    if (fileUrl.startsWith("https://")) {
      this.saveUrl(fileUrl);
    }
    if (fileUrl.startsWith("https://") && !element.hasAttribute(crossOriginAttribute2)) {
      element.setAttribute(crossOriginAttribute2, "anonymous");
    }
  }
  saveUrl = (fileUrl) => {
    const baseUrl = new URL(fileUrl);
    if (!this.urlCache.includes(baseUrl.host) || !this.urlCache.includes(baseUrl.origin)) {
      this.urlCache.push(baseUrl.origin);
    }
  };
  get urls() {
    return this.urlCache.join(" ");
  }
}

// packages/node/src/NodeFile.ts
import { readFile, writeFile } from "node:fs/promises";

class NodeFile {
  path;
  constructor(path) {
    this.path = path;
  }
  async read() {
    return readFile(this.path, { encoding: "utf8" });
  }
  async write(contents) {
    return writeFile(this.path, contents, { encoding: "utf8" });
  }
}

// packages/node/src/NodeHash.ts
import { createHash } from "node:crypto";

class NodeHash {
  algorithm;
  hasher;
  constructor(algorithm) {
    this.hasher = createHash(algorithm);
    this.algorithm = algorithm;
  }
  digest() {
    const hash = this.hasher.digest("base64");
    this.hasher = createHash(this.algorithm);
    return hash;
  }
  update(contentsToHash) {
    this.hasher.update(contentsToHash, "utf8");
  }
}

// packages/node/src/index.ts
var DEFAULT_CSP_POLICY = {
  "base-uri": ["'none'"],
  "default-src": ["'self'"],
  "object-src": ["'none'"]
};
var generateCspPlugin = (options = {}) => {
  const { algorithm = "sha384" } = options;
  const startingPolicy = options.policy ?? { ...DEFAULT_CSP_POLICY };
  let config;
  return {
    name: "generate-csp",
    enforce: "post",
    apply: "build",
    configResolved: ({ root, base, build: { outDir } }) => {
      config = {
        base,
        outDir,
        root
      };
    },
    closeBundle: {
      order: "post",
      handler: async () => {
        const policy = { ...startingPolicy };
        const htmlPath = resolvePath("index.html", config);
        const htmlFile = new NodeFile(htmlPath);
        const html = await htmlFile.read();
        const rewriter = new HTMLRewriter;
        const scriptHandler = new ScriptHandler(algorithm, config, NodeHash, NodeFile);
        const inlineScriptHandler = new InlineScriptHandler(algorithm, config, NodeHash, NodeFile);
        const styleHandler = new StyleHandler(algorithm, config, NodeHash, NodeFile);
        const inlineStyleHandler = new InlineStyleHandler(algorithm, config, NodeHash, NodeFile);
        const newHtml = await rewriter.on("script", scriptHandler).on("script", inlineScriptHandler).on("link", styleHandler).on("style", inlineStyleHandler).transform(new Response(html)).text();
        const csp = buildCsp(policy, {
          scriptHandler,
          inlineScriptHandler,
          styleHandler,
          inlineStyleHandler
        });
        const tag = `<meta http-equiv="Content-Security-Policy" content="${csp}" />`;
        let newHead;
        if (newHtml.includes("<head>")) {
          newHead = newHtml.replace(/<head>/, `<head>${tag}`);
        } else {
          newHead = newHtml.replace(/(<html.*>)/, `$1<head>${tag}`);
        }
        await htmlFile.write(newHead);
      }
    }
  };
};
export {
  generateCspPlugin,
  DEFAULT_CSP_POLICY
};
